package at.jku.ssw.graph;

import at.jku.ssw.Iterator;

/** An unbounded priority queue for vertices based on a priority heap. */
public abstract class VertexPriorityQueue {

	/**
	 * Inserts the value into this priority queue.
	 *
	 * @throws IllegalArgumentException
	 *             if a <code>null</code> value is offered.
	 */
	public abstract void push(Vertex value);

	/**
	 * Retrieves and removes the head of this priority queue.
	 * 
	 * @throws NoSuchElementException
	 *             if the queue is empty
	 */
	public abstract Vertex take();

	/**
	 * Retrieves the head of this priority queue without removing it.
	 * 
	 * @throws NoSuchElementException
	 *             if the queue is empty
	 */
	public abstract Vertex peek();

	/**
	 * Returns <code>true</code> if this queue contains the specified value.
	 *
	 * @throws IllegalArgumentException
	 *             if the specified value is <code>null</code>.
	 */
	public abstract boolean contains(Vertex value);

	/**
	 * Removes all instances of the specified object from this queue, if it is
	 * present. Returns <code>true</code> if and only if the queue contained the
	 * specified value (or in other words, if this queue changed as a result of
	 * the call).
	 *
	 * @throws IllegalArgumentException
	 *             if the specified value is <code>null</code>.
	 */
	public abstract boolean remove(Vertex value);

	/**
	 * Returns the number of values in this priority queue.
	 */
	public abstract int size();

	/**
	 * Returns an iterator of the values in this priority queue. The iterator
	 * returns the values in sorted order, from the value with the highest
	 * priority to the value with the lowest priority.
	 */
	public abstract Iterator iterator();

	/**
	 * Performs an up-heap operation on the specified vertex.
	 *
	 * @throws IllegalArgumentException
	 *             if the specified vertex is <code>null</code>.
	 */
	public abstract void upHeap(Vertex v);
}
